# encoding: utf-8
from __future__ import unicode_literals

import re

from .crunchyroll import CrunchyrollIE

from .common import InfoExtractor
from ..compat import compat_HTTPError
from ..utils import (
    ExtractorError,
    int_or_none,
    remove_start,
    xpath_text,
)


class SoompiBaseIE(InfoExtractor):
    def _get_episodes(self, webpage, episode_filter=None):
        episodes = self._parse_json(
            self._search_regex(
                r'VIDEOS\s*=\s*(\[.+?\]);', webpage, 'episodes JSON'),
            None)
        return list(filter(episode_filter, episodes))


class SoompiIE(SoompiBaseIE, CrunchyrollIE):
    IE_NAME = 'soompi'
    _VALID_URL = r'https?://tv\.soompi\.com/(?:en/)?watch/(?P<id>[0-9]+)'
    _TESTS = [{
        'url': 'http://tv.soompi.com/en/watch/29235',
        'info_dict': {
            'id': '29235',
            'ext': 'mp4',
            'title': 'Episode 1096',
            'description': '2015-05-20'
        },
        'params': {
            'skip_download': True,
        },
    }]

    def _get_episode(self, webpage, video_id):
        return self._get_episodes(webpage, lambda x: x['id'] == video_id)[0]

    def _get_subtitles(self, config, video_id):
        sub_langs = {}
        for subtitle in config.findall('./{default}preload/subtitles/subtitle'):
            sub_langs[subtitle.attrib['id']] = subtitle.attrib['title']

        subtitles = {}
        for s in config.findall('./{default}preload/subtitle'):
            lang_code = sub_langs.get(s.attrib['id'])
            if not lang_code:
                continue
            sub_id = s.get('id')
            data = xpath_text(s, './data', 'data')
            iv = xpath_text(s, './iv', 'iv')
            if not id or not iv or not data:
                continue
            subtitle = self._decrypt_subtitles(data, iv, sub_id).decode('utf-8')
            subtitles[lang_code] = self._extract_subtitles(subtitle)
        return subtitles

    def _real_extract(self, url):
        video_id = self._match_id(url)

        try:
            webpage = self._download_webpage(
                url, video_id, 'Downloading episode page')
        except ExtractorError as ee:
            if isinstance(ee.cause, compat_HTTPError) and ee.cause.code == 403:
                webpage = ee.cause.read()
                block_message = self._html_search_regex(
                    r'(?s)<div class="block-message">(.+?)</div>', webpage,
                    'block message', default=None)
                if block_message:
                    raise ExtractorError(block_message, expected=True)
            raise

        formats = []
        config = None
        for format_id in re.findall(r'\?quality=([0-9a-zA-Z]+)', webpage):
            config = self._download_xml(
                'http://tv.soompi.com/en/show/_/%s-config.xml?mode=hls&quality=%s' % (video_id, format_id),
                video_id, 'Downloading %s XML' % format_id)
            m3u8_url = xpath_text(
                config, './{default}preload/stream_info/file',
                '%s m3u8 URL' % format_id)
            if not m3u8_url:
                continue
            formats.extend(self._extract_m3u8_formats(
                m3u8_url, video_id, 'mp4', m3u8_id=format_id))
        self._sort_formats(formats)

        episode = self._get_episode(webpage, video_id)

        title = episode['name']
        description = episode.get('description')
        duration = int_or_none(episode.get('duration'))

        thumbnails = [{
            'id': thumbnail_id,
            'url': thumbnail_url,
        } for thumbnail_id, thumbnail_url in episode.get('img_url', {}).items()]

        subtitles = self.extract_subtitles(config, video_id)

        return {
            'id': video_id,
            'title': title,
            'description': description,
            'thumbnails': thumbnails,
            'duration': duration,
            'formats': formats,
            'subtitles': subtitles
        }


class SoompiShowIE(SoompiBaseIE):
    IE_NAME = 'soompi:show'
    _VALID_URL = r'https?://tv\.soompi\.com/en/shows/(?P<id>[0-9a-zA-Z\-_]+)'
    _TESTS = [{
        'url': 'http://tv.soompi.com/en/shows/liar-game',
        'info_dict': {
            'id': 'liar-game',
            'title': 'Liar Game',
            'description': 'md5:52c02bce0c1a622a95823591d0589b66',
        },
        'playlist_count': 14,
    }]

    def _real_extract(self, url):
        show_id = self._match_id(url)

        webpage = self._download_webpage(
            url, show_id, 'Downloading show page')

        title = remove_start(self._og_search_title(webpage), 'SoompiTV | ')
        description = self._og_search_description(webpage)

        entries = [
            self.url_result('http://tv.soompi.com/en/watch/%s' % episode['id'], 'Soompi')
            for episode in self._get_episodes(webpage)]

        return self.playlist_result(entries, show_id, title, description)



# coding: utf-8
from __future__ import unicode_literals

from .common import InfoExtractor
from ..compat import compat_urllib_parse_unquote
from ..utils import (
    ExtractorError,
    unified_strdate,
    int_or_none,
    qualities,
    unescapeHTML,
)


class OdnoklassnikiIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?(?:odnoklassniki|ok)\.ru/(?:video|web-api/video/moviePlayer)/(?P<id>[\d-]+)'
    _TESTS = [{
        # metadata in JSON
        'url': 'http://ok.ru/video/20079905452',
        'md5': '6ba728d85d60aa2e6dd37c9e70fdc6bc',
        'info_dict': {
            'id': '20079905452',
            'ext': 'mp4',
            'title': 'Культура меняет нас (прекрасный ролик!))',
            'duration': 100,
            'upload_date': '20141207',
            'uploader_id': '330537914540',
            'uploader': 'Виталий Добровольский',
            'like_count': int,
            'age_limit': 0,
        },
        'skip': 'Video has been blocked',
    }, {
        # metadataUrl
        'url': 'http://ok.ru/video/63567059965189-0',
        'md5': '9676cf86eff5391d35dea675d224e131',
        'info_dict': {
            'id': '63567059965189-0',
            'ext': 'mp4',
            'title': 'Девушка без комплексов ...',
            'duration': 191,
            'upload_date': '20150518',
            'uploader_id': '534380003155',
            'uploader': '☭ Андрей Мещанинов ☭',
            'like_count': int,
            'age_limit': 0,
        },
    }, {
        # YouTube embed (metadataUrl, provider == USER_YOUTUBE)
        'url': 'http://ok.ru/video/64211978996595-1',
        'md5': '5d7475d428845cd2e13bae6f1a992278',
        'info_dict': {
            'id': '64211978996595-1',
            'ext': 'mp4',
            'title': 'Космическая среда от 26 августа 2015',
            'description': 'md5:848eb8b85e5e3471a3a803dae1343ed0',
            'duration': 440,
            'upload_date': '20150826',
            'uploader_id': '750099571',
            'uploader': 'Алина П',
            'age_limit': 0,
        },
    }, {
        'url': 'http://ok.ru/web-api/video/moviePlayer/20079905452',
        'only_matching': True,
    }, {
        'url': 'http://www.ok.ru/video/20648036891',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)

        webpage = self._download_webpage(
            'http://ok.ru/video/%s' % video_id, video_id)

        error = self._search_regex(
            r'[^>]+class="vp_video_stub_txt"[^>]*>([^<]+)<',
            webpage, 'error', default=None)
        if error:
            raise ExtractorError(error, expected=True)

        player = self._parse_json(
            unescapeHTML(self._search_regex(
                r'data-options=(?P<quote>["\'])(?P<player>{.+?%s.+?})(?P=quote)' % video_id,
                webpage, 'player', group='player')),
            video_id)

        flashvars = player['flashvars']

        metadata = flashvars.get('metadata')
        if metadata:
            metadata = self._parse_json(metadata, video_id)
        else:
            metadata = self._download_json(
                compat_urllib_parse_unquote(flashvars['metadataUrl']),
                video_id, 'Downloading metadata JSON')

        movie = metadata['movie']
        title = movie['title']
        thumbnail = movie.get('poster')
        duration = int_or_none(movie.get('duration'))

        author = metadata.get('author', {})
        uploader_id = author.get('id')
        uploader = author.get('name')

        upload_date = unified_strdate(self._html_search_meta(
            'ya:ovs:upload_date', webpage, 'upload date', default=None))

        age_limit = None
        adult = self._html_search_meta(
            'ya:ovs:adult', webpage, 'age limit', default=None)
        if adult:
            age_limit = 18 if adult == 'true' else 0

        like_count = int_or_none(metadata.get('likeCount'))

        info = {
            'id': video_id,
            'title': title,
            'thumbnail': thumbnail,
            'duration': duration,
            'upload_date': upload_date,
            'uploader': uploader,
            'uploader_id': uploader_id,
            'like_count': like_count,
            'age_limit': age_limit,
        }

        if metadata.get('provider') == 'USER_YOUTUBE':
            info.update({
                '_type': 'url_transparent',
                'url': movie['contentId'],
            })
            return info

        quality = qualities(('mobile', 'lowest', 'low', 'sd', 'hd'))

        formats = [{
            'url': f['url'],
            'ext': 'mp4',
            'format_id': f['name'],
            'quality': quality(f['name']),
        } for f in metadata['videos']]
        self._sort_formats(formats)

        info['formats'] = formats
        return info



from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..compat import compat_urllib_request
from ..utils import (
    float_or_none,
    unescapeHTML,
)


class TwitterCardIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?twitter\.com/i/cards/tfw/v1/(?P<id>\d+)'
    _TEST = {
        'url': 'https://twitter.com/i/cards/tfw/v1/560070183650213889',
        'md5': 'a74f50b310c83170319ba16de6955192',
        'info_dict': {
            'id': '560070183650213889',
            'ext': 'mp4',
            'title': 'TwitterCard',
            'thumbnail': 're:^https?://.*\.jpg$',
            'duration': 30.033,
        },
    }

    def _real_extract(self, url):
        video_id = self._match_id(url)

        # Different formats served for different User-Agents
        USER_AGENTS = [
            'Mozilla/5.0 (X11; Linux x86_64; rv:10.0) Gecko/20150101 Firefox/20.0 (Chrome)',  # mp4
            'Mozilla/5.0 (Windows NT 5.2; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0',  # webm
        ]

        config = None
        formats = []
        for user_agent in USER_AGENTS:
            request = compat_urllib_request.Request(url)
            request.add_header('User-Agent', user_agent)
            webpage = self._download_webpage(request, video_id)

            config = self._parse_json(
                unescapeHTML(self._search_regex(
                    r'data-player-config="([^"]+)"', webpage, 'data player config')),
                video_id)

            video_url = config['playlist'][0]['source']

            f = {
                'url': video_url,
            }

            m = re.search(r'/(?P<width>\d+)x(?P<height>\d+)/', video_url)
            if m:
                f.update({
                    'width': int(m.group('width')),
                    'height': int(m.group('height')),
                })
            formats.append(f)
        self._sort_formats(formats)

        thumbnail = config.get('posterImageUrl')
        duration = float_or_none(config.get('duration'))

        return {
            'id': video_id,
            'title': 'TwitterCard',
            'thumbnail': thumbnail,
            'duration': duration,
            'formats': formats,
        }



# coding: utf-8
from __future__ import unicode_literals

from .common import InfoExtractor


class TF1IE(InfoExtractor):
    """TF1 uses the wat.tv player."""
    _VALID_URL = r'http://(?:(?:videos|www|lci)\.tf1|www\.tfou)\.fr/.*?-(?P<id>\d+)(?:-\d+)?\.html'
    _TESTS = [{
        'url': 'http://videos.tf1.fr/auto-moto/citroen-grand-c4-picasso-2013-presentation-officielle-8062060.html',
        'info_dict': {
            'id': '10635995',
            'ext': 'mp4',
            'title': 'Citroën Grand C4 Picasso 2013 : présentation officielle',
            'description': 'Vidéo officielle du nouveau Citroën Grand C4 Picasso, lancé à l\'automne 2013.',
        },
        'params': {
            # Sometimes wat serves the whole file with the --test option
            'skip_download': True,
        },
    }, {
        'url': 'http://www.tfou.fr/chuggington/videos/le-grand-mysterioso-chuggington-7085291-739.html',
        'info_dict': {
            'id': '12043945',
            'ext': 'mp4',
            'title': 'Le grand Mystérioso - Chuggington',
            'description': 'Le grand Mystérioso - Emery rêve qu\'un article lui soit consacré dans le journal.',
            'upload_date': '20150103',
        },
        'params': {
            # Sometimes wat serves the whole file with the --test option
            'skip_download': True,
        },
    }, {
        'url': 'http://www.tf1.fr/tf1/koh-lanta/videos/replay-koh-lanta-22-mai-2015.html',
        'only_matching': True,
    }, {
        'url': 'http://lci.tf1.fr/sept-a-huit/videos/sept-a-huit-du-24-mai-2015-8611550.html',
        'only_matching': True,
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)
        embed_url = self._html_search_regex(
            r'["\'](https?://www.wat.tv/embedframe/.*?)["\']', webpage, 'embed url')
        embed_page = self._download_webpage(embed_url, video_id,
                                            'Downloading embed player page')
        wat_id = self._search_regex(r'UVID=(.*?)&', embed_page, 'wat id')
        wat_info = self._download_json(
            'http://www.wat.tv/interface/contentv3/%s' % wat_id, video_id)
        return self.url_result(wat_info['media']['url'], 'Wat')



# encoding: utf-8
from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..utils import (
    ExtractorError,
    float_or_none,
    xpath_text,
)


class MegaVideozIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?megavideoz\.eu/video/(?P<id>[^/]+)(?:/(?P<display_id>[^/]+))?'
    _TEST = {
        'url': 'http://megavideoz.eu/video/WM6UB919XMXH/SMPTE-Universal-Film-Leader',
        'info_dict': {
            'id': '48723',
            'display_id': 'SMPTE-Universal-Film-Leader',
            'ext': 'mp4',
            'title': 'SMPTE Universal Film Leader',
            'thumbnail': 're:https?://.*?\.jpg',
            'duration': 10.93,
        }
    }

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        video_id = mobj.group('id')
        display_id = mobj.group('display_id') or video_id

        webpage = self._download_webpage(url, display_id)

        if any(p in webpage for p in ('>Video Not Found<', '>404 Error<')):
            raise ExtractorError('Video %s does not exist' % video_id, expected=True)

        config = self._download_xml(
            self._search_regex(
                r"var\s+cnf\s*=\s*'([^']+)'", webpage, 'cnf url'),
            display_id)

        video_url = xpath_text(config, './file', 'video url', fatal=True)
        title = xpath_text(config, './title', 'title', fatal=True)
        thumbnail = xpath_text(config, './image', 'thumbnail')
        duration = float_or_none(xpath_text(config, './duration', 'duration'))
        video_id = xpath_text(config, './mediaid', 'video id') or video_id

        return {
            'id': video_id,
            'display_id': display_id,
            'url': video_url,
            'title': title,
            'thumbnail': thumbnail,
            'duration': duration
        }



# coding: utf-8
from __future__ import unicode_literals

import re
import itertools
import json
import xml.etree.ElementTree as ET

from .common import InfoExtractor
from ..utils import (
    int_or_none,
    unified_strdate,
    ExtractorError,
)


class BiliBiliIE(InfoExtractor):
    _VALID_URL = r'http://www\.bilibili\.(?:tv|com)/video/av(?P<id>[0-9]+)/'

    _TESTS = [{
        'url': 'http://www.bilibili.tv/video/av1074402/',
        'md5': '2c301e4dab317596e837c3e7633e7d86',
        'info_dict': {
            'id': '1074402_part1',
            'ext': 'flv',
            'title': '【金坷垃】金泡沫',
            'duration': 308,
            'upload_date': '20140420',
            'thumbnail': 're:^https?://.+\.jpg',
        },
    }, {
        'url': 'http://www.bilibili.com/video/av1041170/',
        'info_dict': {
            'id': '1041170',
            'title': '【BD1080P】刀语【诸神&异域】',
        },
        'playlist_count': 9,
    }]

    def _real_extract(self, url):
        video_id = self._match_id(url)
        webpage = self._download_webpage(url, video_id)

        if '(此视频不存在或被删除)' in webpage:
            raise ExtractorError(
                'The video does not exist or was deleted', expected=True)

        if '>你没有权限浏览！ 由于版权相关问题 我们不对您所在的地区提供服务<' in webpage:
            raise ExtractorError(
                'The video is not available in your region due to copyright reasons',
                expected=True)

        video_code = self._search_regex(
            r'(?s)<div itemprop="video".*?>(.*?)</div>', webpage, 'video code')

        title = self._html_search_meta(
            'media:title', video_code, 'title', fatal=True)
        duration_str = self._html_search_meta(
            'duration', video_code, 'duration')
        if duration_str is None:
            duration = None
        else:
            duration_mobj = re.match(
                r'^T(?:(?P<hours>[0-9]+)H)?(?P<minutes>[0-9]+)M(?P<seconds>[0-9]+)S$',
                duration_str)
            duration = (
                int_or_none(duration_mobj.group('hours'), default=0) * 3600 +
                int(duration_mobj.group('minutes')) * 60 +
                int(duration_mobj.group('seconds')))
        upload_date = unified_strdate(self._html_search_meta(
            'uploadDate', video_code, fatal=False))
        thumbnail = self._html_search_meta(
            'thumbnailUrl', video_code, 'thumbnail', fatal=False)

        cid = self._search_regex(r'cid=(\d+)', webpage, 'cid')

        entries = []

        lq_page = self._download_webpage(
            'http://interface.bilibili.com/v_cdn_play?appkey=1&cid=%s' % cid,
            video_id,
            note='Downloading LQ video info'
        )
        try:
            err_info = json.loads(lq_page)
            raise ExtractorError(
                'BiliBili said: ' + err_info['error_text'], expected=True)
        except ValueError:
            pass

        lq_doc = ET.fromstring(lq_page)
        lq_durls = lq_doc.findall('./durl')

        hq_doc = self._download_xml(
            'http://interface.bilibili.com/playurl?appkey=1&cid=%s' % cid,
            video_id,
            note='Downloading HQ video info',
            fatal=False,
        )
        if hq_doc is not False:
            hq_durls = hq_doc.findall('./durl')
            assert len(lq_durls) == len(hq_durls)
        else:
            hq_durls = itertools.repeat(None)

        i = 1
        for lq_durl, hq_durl in zip(lq_durls, hq_durls):
            formats = [{
                'format_id': 'lq',
                'quality': 1,
                'url': lq_durl.find('./url').text,
                'filesize': int_or_none(
                    lq_durl.find('./size'), get_attr='text'),
            }]
            if hq_durl is not None:
                formats.append({
                    'format_id': 'hq',
                    'quality': 2,
                    'ext': 'flv',
                    'url': hq_durl.find('./url').text,
                    'filesize': int_or_none(
                        hq_durl.find('./size'), get_attr='text'),
                })
            self._sort_formats(formats)

            entries.append({
                'id': '%s_part%d' % (video_id, i),
                'title': title,
                'formats': formats,
                'duration': duration,
                'upload_date': upload_date,
                'thumbnail': thumbnail,
            })

            i += 1

        return {
            '_type': 'multi_video',
            'entries': entries,
            'id': video_id,
            'title': title
        }



from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..utils import (
    determine_ext,
    int_or_none,
    js_to_json,
    parse_iso8601,
    remove_end,
)


class ClipfishIE(InfoExtractor):
    _VALID_URL = r'https?://(?:www\.)?clipfish\.de/(?:[^/]+/)+video/(?P<id>[0-9]+)'
    _TEST = {
        'url': 'http://www.clipfish.de/special/game-trailer/video/3966754/fifa-14-e3-2013-trailer/',
        'md5': '79bc922f3e8a9097b3d68a93780fd475',
        'info_dict': {
            'id': '3966754',
            'ext': 'mp4',
            'title': 'FIFA 14 - E3 2013 Trailer',
            'timestamp': 1370938118,
            'upload_date': '20130611',
            'duration': 82,
        }
    }

    def _real_extract(self, url):
        video_id = self._match_id(url)

        webpage = self._download_webpage(url, video_id)

        video_info = self._parse_json(
            js_to_json(self._html_search_regex(
                '(?s)videoObject\s*=\s*({.+?});', webpage, 'video object')),
            video_id)

        formats = []
        for video_url in re.findall(r'var\s+videourl\s*=\s*"([^"]+)"', webpage):
            ext = determine_ext(video_url)
            if ext == 'm3u8':
                formats.append({
                    'url': video_url.replace('de.hls.fra.clipfish.de', 'hls.fra.clipfish.de'),
                    'ext': 'mp4',
                    'format_id': 'hls',
                })
            else:
                formats.append({
                    'url': video_url,
                    'format_id': ext,
                })
        self._sort_formats(formats)

        title = remove_end(self._og_search_title(webpage), ' - Video')
        thumbnail = self._og_search_thumbnail(webpage)
        duration = int_or_none(video_info.get('length'))
        timestamp = parse_iso8601(self._html_search_meta('uploadDate', webpage, 'upload date'))

        return {
            'id': video_id,
            'title': title,
            'formats': formats,
            'thumbnail': thumbnail,
            'duration': duration,
            'timestamp': timestamp,
        }



from __future__ import unicode_literals

import re

from .common import InfoExtractor
from ..utils import (
    clean_html,
    determine_ext,
    ExtractorError,
)


class FKTVIE(InfoExtractor):
    IE_NAME = 'fernsehkritik.tv'
    _VALID_URL = r'http://(?:www\.)?fernsehkritik\.tv/folge-(?P<id>[0-9]+)(?:/.*)?'

    _TEST = {
        'url': 'http://fernsehkritik.tv/folge-1',
        'md5': '21f0b0c99bce7d5b524eb1b17b1c6d79',
        'info_dict': {
            'id': '1',
            'ext': 'mp4',
            'title': 'Folge 1 vom 10. April 2007',
            'thumbnail': 're:^https?://.*\.jpg$',
        },
    }

    def _real_extract(self, url):
        episode = self._match_id(url)

        webpage = self._download_webpage(
            'http://fernsehkritik.tv/folge-%s/play' % episode, episode)
        title = clean_html(self._html_search_regex(
            '<h3>([^<]+)</h3>', webpage, 'title'))
        matches = re.search(
            r'(?s)<video(?:(?!poster)[^>])+(?:poster="([^"]+)")?[^>]*>(.*)</video>',
            webpage)
        if matches is None:
            raise ExtractorError('Unable to extract the video')

        poster, sources = matches.groups()
        if poster is None:
            self.report_warning('unable to extract thumbnail')

        urls = re.findall(r'<source[^>]+src="([^"]+)"', sources)
        formats = [{
            'url': furl,
            'format_id': determine_ext(furl),
        } for furl in urls]
        return {
            'id': episode,
            'title': title,
            'formats': formats,
            'thumbnail': poster,
        }
